package com.networksByteDesign.util;

import org.apache.tools.ant.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.types.*;
import java.util.Iterator;
import java.util.HashSet;
import org.apache.tools.ant.util.FileNameMapper;
 
public class MissingTestsTask extends MatchingTask
{
    public void execute()
    {
        // Get a handle on the DirectoryScanner for each fileset
        DirectoryScanner dsFiles =
            filesetFiles_.getDirectoryScanner(project);
        DirectoryScanner dsTests =
            filesetTests_.getDirectoryScanner(project);
        
        // Get a list of the included files for each fileset
        String[] fileList = dsFiles.getIncludedFiles();
        String[] testList = dsTests.getIncludedFiles();
        
        // Create the HashSets
        HashSet testSet = new HashSet();
        HashSet fileSet = new HashSet();
        
        // Get a FileNameMapper from the mapperElement that
        // was passed in from the buildfile
        FileNameMapper mapper = null;
        if (mapperElement_ != null)
        {
            mapper = mapperElement_.getImplementation();
        }
        
        try
        {
            // Perform the file name mapping on the list of Test
            // filenames and load them into a HashSet
            for (int i = 0; i < testList.length; i++)
            {
                String[] results = mapper.mapFileName(testList[i]);
                testSet.add(results[0]);
            }
            
            // Load the names of the files from the 
            // FilesFileset into a HashSet
            for (int k=0; k<fileList.length; k++)
            {
                fileSet.add(fileList[k]);
            }
            
            // Remove all of the mapped names in the Test 
            // Names HashSet from the File Names HashSet
            fileSet.removeAll(testSet);
            
            // The remaining files are those that don't have 
            // a corresponding test.  Use an interator to get the 
            // names, and then print the list.
            Iterator testIter = fileSet.iterator();
            System.out.println("The following classes do not have tests:");
            while (testIter.hasNext())
            {
                System.out.println(testIter.next());
            }
        }
        catch (Exception e)
        {
            System.err.println(e);
        }
    }
    
    public void addTestsFileset(FileSet set)
    {
        filesetTests_ = set;
    }
    
    public void addFilesFileset(FileSet set)
    {
        filesetFiles_ = set;
    }
    
    public Mapper createMapper() throws BuildException
    {
        if (mapperElement_ != null)
        {
            throw new BuildException("Cannot define more than one mapper",
                                     location);
        }
        mapperElement_ = new Mapper(project);
        return mapperElement_;
    }
    
    private Mapper   mapperElement_;
    private FileSet  filesetTests_;
    private FileSet  filesetFiles_;
}
